/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	gdt.cpp
	Sets up and handles the Global Descriptor Table.

******************************************************************************/

#include "types.h"
#include "memory.h"
#include "gdt.h"

extern "C" void GDT__Flush(void* GDT);

void GDT::Init()
{
	//GOut->Print("Init 1 GDT!\n"); HackSleep(2000);

	// Sets the special IDT pointer up.
	Pointer.Limit = (sizeof(Entries)) - 1;
	Pointer.Base = &Entries;

	// Our NULL descriptor.
	SetGate(0, NULL, 0, 0, 0);

	// The second entry is our Code Segment. The base address
	// is 0, the limit is 4GBytes, it uses 4KByte granularity,
	// uses 32-bit opcodes, and is a Code Segment descriptor.
	SetGate(1, NULL, 0xFFFFFFFF, 0x9A, 0xCF);

    // The third entry is our Data Segment. It's exactly the
    // same as our code segment, but the descriptor type in
    // this entry's access byte says it's a Data Segment.
	SetGate(2, NULL, 0xFFFFFFFF, 0x92, 0xCF);
	SetGate(3, NULL, 0xFFFFFFFF, 0xFA, 0xCF); // User mode code segment
	SetGate(4, NULL, 0xFFFFFFFF, 0xF2, 0xCF); // User mode data segment

	GDT__Flush(&Pointer);
}

void GDT::SetGate(size_t Num, void* BaseP, size_t Limit, uint8_t Access, uint8_t Granularity)
{
	uintptr_t Base = (uintptr_t) BaseP;

	Entries[Num].BaseLow = Base & 0xFFFF;
	Entries[Num].BaseMiddle = (Base >> 16) & 0xFF;
	Entries[Num].BaseHigh = (Base >> 24) & 0xFF;
	Entries[Num].LimitLow = Limit & 0xFFFF;
	Entries[Num].Granularity = ((Limit >> 16) & 0x0F) | (Granularity & 0xF0);
	Entries[Num].Access = Access;
}

