/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	idt.cpp
	Sets up and handles the Interrupt Descriptor Table.

******************************************************************************/

#include "types.h"
#include "memory.h"
#include "idt.h"
#include "isr.h"
#include "kernel.h"

extern "C" void IDT__Flush(void* IDT);

void IDT::Init()
{
	// Sets the special IDT pointer up.
	Pointer.Limit = (sizeof(Entries)) - 1;
	Pointer.Base = &Entries;

	// Clear out the entire IDT, initializing it to zeros.
	MemorySet(&Entries, 0, sizeof(Entries));

	// Initialize the Interrupt Service Routines (ISR).
	GISR->Init();

	// Points the processor's internal register to the new IDT.
	IDT__Flush(&Pointer);
}

void IDT::SetGate(size_t Num, void* BaseP, uint16_t Sel, uint16_t Flags)
{
	uintptr_t Base = (uintptr_t) BaseP;

	Entries[Num].BaseLow = Base & 0xFFFF;
	Entries[Num].BaseHigh = (Base >> 16) & 0xFFFF;
	Entries[Num].Sel = Sel;
	Entries[Num].Always0 = 0; // Surprise.
	Entries[Num].Flags = Flags;
}

