/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	isr-asm.s
	Sets up and handles the Interrupt Service Routines.

******************************************************************************/

.globl ISR__ISR0
.globl ISR__ISR1
.globl ISR__ISR2
.globl ISR__ISR3
.globl ISR__ISR4
.globl ISR__ISR5
.globl ISR__ISR6
.globl ISR__ISR7
.globl ISR__ISR8
.globl ISR__ISR9
.globl ISR__ISR10
.globl ISR__ISR11
.globl ISR__ISR12
.globl ISR__ISR13
.globl ISR__ISR14
.globl ISR__ISR15
.globl ISR__ISR16
.globl ISR__ISR17
.globl ISR__ISR18
.globl ISR__ISR19
.globl ISR__ISR20
.globl ISR__ISR21
.globl ISR__ISR22
.globl ISR__ISR23
.globl ISR__ISR24
.globl ISR__ISR25
.globl ISR__ISR26
.globl ISR__ISR27
.globl ISR__ISR28
.globl ISR__ISR29
.globl ISR__ISR30
.globl ISR__ISR31
.section .text

# Division By Zero Exception
.type ISR__ISR0, @function
ISR__ISR0:
	#pushl $HEYHEY
	#call Panic
	cli
	pushl $0
	pushl $0
	jmp ISR__CommonStub

# Debug Exception
.type ISR__ISR1, @function
ISR__ISR1:
	cli
	pushl $0
	pushl $1
	jmp ISR__CommonStub

# Non Maskable Interrupt Exception
.type ISR__ISR2, @function
ISR__ISR2:
	cli
	pushl $0
	pushl $2
	jmp ISR__CommonStub

# Breakpoint Exception
.type ISR__ISR3, @function
ISR__ISR3:
	cli
	pushl $0
	pushl $3
	jmp ISR__CommonStub

# Into Detected Overflow Exception
.type ISR__ISR4, @function
ISR__ISR4:
	cli
	pushl $0
	pushl $4
	jmp ISR__CommonStub

# Out of Bounds Exception
.type ISR__ISR5, @function
ISR__ISR5:
	cli
	pushl $0
	pushl $5
	jmp ISR__CommonStub

# Invalid Opcode Exception
.type ISR__ISR6, @function
ISR__ISR6:
	cli
	pushl $0
	pushl $6
	jmp ISR__CommonStub

# No Coprocessor Exception
.type ISR__ISR7, @function
ISR__ISR7:
	cli
	pushl $0
	pushl $7
	jmp ISR__CommonStub

# Double Fault Exception
.type ISR__ISR8, @function
ISR__ISR8:
	cli
	#pushl $0 # Error code already pushed by CPU.
	pushl $8
	jmp ISR__CommonStub

# Coprocessor Segment Overrun Exception
.type ISR__ISR9, @function
ISR__ISR9:
	cli
	pushl $0
	pushl $9
	jmp ISR__CommonStub

# Bad TSS Exception
.type ISR__ISR10, @function
ISR__ISR10:
	cli
	#pushl $0 # Error code already pushed by CPU.
	pushl $10
	jmp ISR__CommonStub

# Segment Not Present Exception
.type ISR__ISR11, @function
ISR__ISR11:
	cli
	#pushl $0 # Error code already pushed by CPU.
	pushl $11
	jmp ISR__CommonStub

# Stack Fault Exception
.type ISR__ISR12, @function
ISR__ISR12:
	cli
	#pushl $0 # Error code already pushed by CPU.
	pushl $12
	jmp ISR__CommonStub

# General Protection Fault Exception
.type ISR__ISR13, @function
ISR__ISR13:
	cli
	#pushl $0 # Error code already pushed by CPU.
	pushl $13
	jmp ISR__CommonStub

# Page Fault Exception
.type ISR__ISR14, @function
ISR__ISR14:
	cli
	#pushl $0 # Error code already pushed by CPU.
	pushl $14
	jmp ISR__CommonStub

# Unknown Interrupt Exception
.type ISR__ISR15, @function
ISR__ISR15:
	cli
	pushl $0
	pushl $15
	jmp ISR__CommonStub

# Coprocessor Fault Exception
.type ISR__ISR16, @function
ISR__ISR16:
	cli
	pushl $0
	pushl $16
	jmp ISR__CommonStub

# Alignment Check Exception (486+)
.type ISR__ISR17, @function
ISR__ISR17:
	cli
	pushl $0
	pushl $17
	jmp ISR__CommonStub

# Machine Check Exception (Pentium/586+)
.type ISR__ISR18, @function
ISR__ISR18:
	cli
	pushl $0
	pushl $18
	jmp ISR__CommonStub

# Reserved Exception
.type ISR__ISR19, @function
ISR__ISR19:
	cli
	pushl $0
	pushl $19
	jmp ISR__CommonStub

# Reserved Exception
.type ISR__ISR20, @function
ISR__ISR20:
	cli
	pushl $0
	pushl $20
	jmp ISR__CommonStub

# Reserved Exception
.type ISR__ISR21, @function
ISR__ISR21:
	cli
	pushl $0
	pushl $21
	jmp ISR__CommonStub

# Reserved Exception
.type ISR__ISR22, @function
ISR__ISR22:
	cli
	pushl $0
	pushl $22
	jmp ISR__CommonStub

# Reserved Exception
.type ISR__ISR23, @function
ISR__ISR23:
	cli
	pushl $0
	pushl $23
	jmp ISR__CommonStub

# Reserved Exception
.type ISR__ISR24, @function
ISR__ISR24:
	cli
	pushl $0
	pushl $24
	jmp ISR__CommonStub

# Reserved Exception
.type ISR__ISR25, @function
ISR__ISR25:
	cli
	pushl $0
	pushl $25
	jmp ISR__CommonStub

# Reserved Exception
.type ISR__ISR26, @function
ISR__ISR26:
	cli
	pushl $0
	pushl $26
	jmp ISR__CommonStub

# Reserved Exception
.type ISR__ISR27, @function
ISR__ISR27:
	cli
	pushl $0
	pushl $27
	jmp ISR__CommonStub

# Reserved Exception
.type ISR__ISR28, @function
ISR__ISR28:
	cli
	pushl $0
	pushl $28
	jmp ISR__CommonStub

# Reserved Exception
.type ISR__ISR29, @function
ISR__ISR29:
	cli
	pushl $0
	pushl $29
	jmp ISR__CommonStub

# Reserved Exception
.type ISR__ISR30, @function
ISR__ISR30:
	cli
	pushl $0
	pushl $30
	jmp ISR__CommonStub

# Reserved Exception
.type ISR__ISR31, @function
ISR__ISR31:
	cli
	pushl $0
	pushl $31
	jmp ISR__CommonStub

.type ISR__CommonStub, @function
ISR__CommonStub:

	pusha

	push %ds
	push %es
	push %fs
	push %gs

	mov $0x10, %ax # Load the kernel data segment descriptor.
	mov %ax, %ds
	mov %ax, %es
	mov %ax, %fs
	mov %ax, %gs

	call ISR__FaultHandler

	pop %eax # Reload the original data segment descriptor.
	pop %gs
	pop %fs
	pop %es
	pop %ds

	popa
	add $8, %esp # Cleans up the pushed error code and pushed ISR number
	sti
	iret # pops 5 things at once: CS, EIP, EFLAGS, SS, and ESP.

