/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	isr.cpp
	Sets up and handles the Interrupt Service Routines.

******************************************************************************/

#include "types.h"
#include "idt.h"
#include "isr.h"
#include "panic.h"
#include "kernel.h"

extern "C" void ISR__ISR0();
extern "C" void ISR__ISR1();
extern "C" void ISR__ISR2();
extern "C" void ISR__ISR3();
extern "C" void ISR__ISR4();
extern "C" void ISR__ISR5();
extern "C" void ISR__ISR6();
extern "C" void ISR__ISR7();
extern "C" void ISR__ISR8();
extern "C" void ISR__ISR9();
extern "C" void ISR__ISR10();
extern "C" void ISR__ISR11();
extern "C" void ISR__ISR12();
extern "C" void ISR__ISR13();
extern "C" void ISR__ISR14();
extern "C" void ISR__ISR15();
extern "C" void ISR__ISR16();
extern "C" void ISR__ISR17();
extern "C" void ISR__ISR18();
extern "C" void ISR__ISR19();
extern "C" void ISR__ISR20();
extern "C" void ISR__ISR21();
extern "C" void ISR__ISR22();
extern "C" void ISR__ISR23();
extern "C" void ISR__ISR24();
extern "C" void ISR__ISR25();
extern "C" void ISR__ISR26();
extern "C" void ISR__ISR27();
extern "C" void ISR__ISR28();
extern "C" void ISR__ISR29();
extern "C" void ISR__ISR30();
extern "C" void ISR__ISR31();

const char* IDR__Messages[] = 
{
	"Division By Zero Exception",
	"Debug Exception",
	"Non Maskable Interrupt Exception",
	"Breakpoint Exception",
	"Into Detected Overflow Exception",
	"Out of Bounds Exception",
	"Invalid Opcode Exception",
	"No Coprocessor Exception",
	"Double Fault Exception",
	"Coprocessor Segment Overrun Exception",
	"Bad TSS Exception",
	"Segment Not Present Exception",
	"Stack Fault Exception",
	"General Protection Fault Exception",
	"Page Fault Exception",
	"Unknown Interrupt Exception",
	"Coprocessor Fault Exception",
	"Alignment Check Exception (486+)",
	"Machine Check Exception (Pentium/586+)",
	"Reserved Exception",
	"Reserved Exception",
	"Reserved Exception",
	"Reserved Exception",
	"Reserved Exception",
	"Reserved Exception",
	"Reserved Exception",
	"Reserved Exception",
	"Reserved Exception",
	"Reserved Exception",
	"Reserved Exception",
	"Reserved Exception",
	"Reserved Exception",
};

void ISR::Init()
{
	uint16_t Sel = 0x08;
	uint16_t Flags = 0x8E; // 0x80 | 0xD | 0x1;

	GIDT->SetGate(0, (void*) ISR__ISR0, Sel, Flags);
	GIDT->SetGate(1, (void*) ISR__ISR1, Sel, Flags);
	GIDT->SetGate(2, (void*) ISR__ISR2, Sel, Flags);
	GIDT->SetGate(3, (void*) ISR__ISR3, Sel, Flags);
	GIDT->SetGate(4, (void*) ISR__ISR4, Sel, Flags);
	GIDT->SetGate(5, (void*) ISR__ISR5, Sel, Flags);
	GIDT->SetGate(6, (void*) ISR__ISR6, Sel, Flags);
	GIDT->SetGate(7, (void*) ISR__ISR7, Sel, Flags);
	GIDT->SetGate(8, (void*) ISR__ISR8, Sel, Flags);
	GIDT->SetGate(9, (void*) ISR__ISR9, Sel, Flags);
	GIDT->SetGate(10, (void*) ISR__ISR10, Sel, Flags);
	GIDT->SetGate(11, (void*) ISR__ISR11, Sel, Flags);
	GIDT->SetGate(12, (void*) ISR__ISR12, Sel, Flags);
	GIDT->SetGate(13, (void*) ISR__ISR13, Sel, Flags);
	GIDT->SetGate(14, (void*) ISR__ISR14, Sel, Flags);
	GIDT->SetGate(15, (void*) ISR__ISR15, Sel, Flags);
	GIDT->SetGate(16, (void*) ISR__ISR16, Sel, Flags);
	GIDT->SetGate(17, (void*) ISR__ISR17, Sel, Flags);
	GIDT->SetGate(18, (void*) ISR__ISR18, Sel, Flags);
	GIDT->SetGate(19, (void*) ISR__ISR19, Sel, Flags);
	GIDT->SetGate(20, (void*) ISR__ISR20, Sel, Flags);
	GIDT->SetGate(21, (void*) ISR__ISR21, Sel, Flags);
	GIDT->SetGate(22, (void*) ISR__ISR22, Sel, Flags);
	GIDT->SetGate(23, (void*) ISR__ISR23, Sel, Flags);
	GIDT->SetGate(24, (void*) ISR__ISR24, Sel, Flags);
	GIDT->SetGate(25, (void*) ISR__ISR25, Sel, Flags);
	GIDT->SetGate(26, (void*) ISR__ISR26, Sel, Flags);
	GIDT->SetGate(27, (void*) ISR__ISR27, Sel, Flags);
	GIDT->SetGate(28, (void*) ISR__ISR28, Sel, Flags);
	GIDT->SetGate(29, (void*) ISR__ISR29, Sel, Flags);
	GIDT->SetGate(30, (void*) ISR__ISR30, Sel, Flags);
	GIDT->SetGate(31, (void*) ISR__ISR31, Sel, Flags);
}

// This defines what the stack looks like after an ISR was running
struct ISR__Regs
{
    unsigned int gs, fs, es, ds; // Pushed the segs last
    unsigned int edi, esi, ebp, esp, ebx, edx, ecx, eax; // Pushed by 'pusha'.
    unsigned int InterruptNumber, ErrorCode; // our 'Push byte #' and ecodes do this.
    unsigned int eip, cs, eflags, useresp, ss; // Pushed by the processor automatically.
};	

extern "C" void ISR__FaultHandler(ISR__Regs Regs)
{
    // Is this a fault whose number is from 0 to 31?
    if ( Regs.InterruptNumber < 32 )
    {
       Panic(IDR__Messages[Regs.InterruptNumber]);
    }
}

