/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	paging.h
	Handles paging and virtual memory.

******************************************************************************/

#include "multiboot.h"
#include "types.h"
#include "paging.h"
#include "pageallocator.h"
#include "panic.h"
#include "kernel.h"

// These are extern assembly functions that should not be called from other
// places than this file, this avoids issues!
extern "C" void Paging__Enable(size_t* PageDirectoryPhysicalAddress);
extern "C" void Paging__Disable();

void Paging::Init(size_t KernelStart, size_t KernelSize)
{
	// Allocate a new page to store our page directory in.
	PageDirectory = (size_t*) GPageAllocator->AllocatePage();

	if ( PageDirectory == NULL ) { Panic("paging.cpp: Insufficient memory available to initialize paging!"); }

	// Reset the page directory.
	for ( size_t I = 0; I < 1024; I++ ) { PageDirectory[I] = 0; }

	// Then identity map our page directory.
	Identity(PageDirectory);

	// Now, let's identity map our kernel.
	for ( size_t I = KernelStart; I < KernelStart + KernelSize; I += 4096 ) { Identity((void*) I); }

	// The page allocator might wnat to identity map itself too.
	GPageAllocator->IdentityMap();

	// TODO: We are not ready to implement virtual memory yet!
	return;

	// Now that the initial page directory has been set up, enable paging.
	Paging__Enable(PageDirectory);
}

void Paging::Identity(void* Physical)
{
	// Alright, first we need to find which page dir table we need.
	size_t DirIndex = ((size_t) Physical) / 4096;
	
	// Look up the entry in the page dir.
	size_t Entry = PageDirectory[DirIndex];

	// Check if we have a page.
	
	// TODO: Incomplete!
}


