/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	paging.h
	Handles paging and virtual memory.

******************************************************************************/

#ifndef SORTIX_PAGING_H
#define SORTIX_PAGING_H

#define PAGE_FLAG_PRESENT (1<<0)
#define PAGE_FLAG_USER_WRITABLE (1<<1)
#define PAGE_FLAG_USER_ACCESSABLE (1<<2)
#define PAGE_FLAG_WRITE_THROUGH (1<<3)
#define PAGE_FLAG_DISABLE_CACHE (1<<4)
#define PAGE_FLAG_ACCESSED (1<<5)
#define PAGE_FLAG_4MIB (1<<6)
#define PAGE_FLAG_UNKNOWN (1<<7)

class Paging
{
private:
	size_t* PageDirectory;

public:
	void Init(size_t KernelStart, size_t KernelSize);

public:
	void Identity(void* Physical);
	void Map(void* Physical, void* Virtual, uint32_t Flags);
	void Unmap(void* Virtual);

public:
	void* Virtual2Physical(void* Virtual);
	bool IsUserspaceWritable(void* Virtual, size_t Length);
	bool IsUserspaceReadable(void* Virtual, size_t Length);

public:
	uint32_t GetFlags(void* Virtual);

};

#endif

