/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	vidmemtextscreen.cpp
	Writes directly to the 80x25 video memory and displays colorable text.

******************************************************************************/

#include "types.h"
#include "itextscreen.h"
#include "vidmemtextscreen.h"

clrchar* VidMem = (clrchar*) 0xb8000;

VidMemTextScreen::VidMemTextScreen()
{
	Width = 80;
	Height = 25;
	Color = (COLOR8_BLACK << 12) + (COLOR8_LIGHT_GREY << 8);
	Pos = 0;

	// TODO: A memset would be faster, but we haven't implemented that yet.
	// The compiler will probably optimize this regardless.
	for ( size_t I = 0; I < Width*Height; I++ )
	{
		VidMem[I] = ' ' | Color;
	}
}

ITextScreen::Format VidMemTextScreen::GetFormat() { return BG4FG4TEXT8; }

size_t VidMemTextScreen::GetWidth() { return Width; }
size_t VidMemTextScreen::GetHeight() { return Height; }

void VidMemTextScreen::Seek(size_t X, size_t Y)
{
	Y += X / Width;
	X %= Width;
	Y %= Height;

	Pos = Y * Width + X;
}

void VidMemTextScreen::WriteRaw(const void* Data, size_t DataSize)
{
	const clrchar* Input = (const clrchar*) Data;
	size_t InputLength = DataSize / sizeof(clrchar);
	
	size_t MemLength = Width * Height;

	for ( size_t I = 0; I < InputLength; I++ )
	{
		VidMem[Pos] = Input[I];
		Pos = (Pos + 1) % MemLength;
	}
}

bool VidMemTextScreen::Write(ITextScreen::Format F, const void* Data, size_t DataSize)
{
	if ( F == BG4FG4TEXT8 ) { WriteRaw(Data, DataSize); return true; }
	if ( F != TEXT8 ) { return false; }
	
	const char* Input = (const char*) Data;
	size_t InputLength = DataSize;
	
	size_t MemLength = Width * Height;

	for ( size_t I = 0; I < InputLength; I++ )
	{
		VidMem[Pos] = Input[I] | Color;
		Pos = (Pos + 1) % MemLength;
	}

	return true;
}

