/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	irq.cpp
	Sets up and handles the Interrupt Requests.

******************************************************************************/

#include "types.h"
#include "memory.h"
#include "idt.h"
#include "irq.h"
#include "panic.h"
#include "kernel.h"

extern "C" void IRQ__IRQ0();
extern "C" void IRQ__IRQ1();
extern "C" void IRQ__IRQ2();
extern "C" void IRQ__IRQ3();
extern "C" void IRQ__IRQ4();
extern "C" void IRQ__IRQ5();
extern "C" void IRQ__IRQ6();
extern "C" void IRQ__IRQ7();
extern "C" void IRQ__IRQ8();
extern "C" void IRQ__IRQ9();
extern "C" void IRQ__IRQ10();
extern "C" void IRQ__IRQ11();
extern "C" void IRQ__IRQ12();
extern "C" void IRQ__IRQ13();
extern "C" void IRQ__IRQ14();
extern "C" void IRQ__IRQ15();

void IRQ::SetHandler(size_t Index, IRQ__Handler Handler)
{
	Handlers[Index] = Handler;
}

void IRQ::Init()
{
	// Reset the IRQ handlers.
	MemorySet(Handlers, 0, sizeof(Handlers));

	uint16_t Sel = 0x08;
	uint16_t Flags = 0x80 | 0xD | 0x1;

	GIDT->SetGate(32, (void*) IRQ__IRQ0, Sel, Flags);
	GIDT->SetGate(33, (void*) IRQ__IRQ1, Sel, Flags);
	GIDT->SetGate(34, (void*) IRQ__IRQ2, Sel, Flags);
	GIDT->SetGate(35, (void*) IRQ__IRQ3, Sel, Flags);
	GIDT->SetGate(36, (void*) IRQ__IRQ4, Sel, Flags);
	GIDT->SetGate(37, (void*) IRQ__IRQ5, Sel, Flags);
	GIDT->SetGate(38, (void*) IRQ__IRQ6, Sel, Flags);
	GIDT->SetGate(39, (void*) IRQ__IRQ7, Sel, Flags);
	GIDT->SetGate(40, (void*) IRQ__IRQ8, Sel, Flags);
	GIDT->SetGate(41, (void*) IRQ__IRQ9, Sel, Flags);
	GIDT->SetGate(42, (void*) IRQ__IRQ10, Sel, Flags);
	GIDT->SetGate(43, (void*) IRQ__IRQ11, Sel, Flags);
	GIDT->SetGate(44, (void*) IRQ__IRQ12, Sel, Flags);
	GIDT->SetGate(45, (void*) IRQ__IRQ13, Sel, Flags);
	GIDT->SetGate(46, (void*) IRQ__IRQ14, Sel, Flags);
	GIDT->SetGate(47, (void*) IRQ__IRQ15, Sel, Flags);
}

void IRQ::FireHandler(IRQ__Regs* Regs)
{
	// Execute the handler for this IRQ.
	if ( Handlers[Regs->InterruptNumber] != NULL ) { Handlers[Regs->InterruptNumber](Regs); }
}

extern "C" void IRQ__Handle(IRQ__Regs Regs)
{
	Panic("IRQ!");

	// Todo: Handle the (possibly) spurious IRQ 7 and 15 here.

	// Send an EOI (end of interrupt) signal to the PICs if this interrupt involved the slave.
	if ( Regs.InterruptNumber >= 40 ) { WriteByteToPort(0xA0, 0x20); }
	// Send reset signal to master. (As well as slave, if necessary).
	WriteByteToPort(0x20, 0x20);
	
	GIRQ->FireHandler(&Regs);
}

