/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	irq.h
	Sets up and handles the Interrupt Requests.

******************************************************************************/

#ifndef SORTIX_IRQ_H
#define SORTIX_IRQ_H

#define IRQ0 32
#define IRQ1 33
#define IRQ2 34
#define IRQ3 35
#define IRQ4 36
#define IRQ5 37
#define IRQ6 38
#define IRQ7 39
#define IRQ8 40
#define IRQ9 41
#define IRQ10 42
#define IRQ11 43
#define IRQ12 44
#define IRQ13 45
#define IRQ14 46
#define IRQ15 47

// This defines what the stack looks like after an IRQ was running
struct IRQ__Regs
{
	unsigned int gs, fs, es, ds; // Pushed the segs last
	unsigned int edi, esi, ebp, esp, ebx, edx, ecx, eax; // Pushed by 'pusha'.
	unsigned int InterruptNumber, ErrorCode; // our 'Push byte #' and ecodes do this.
	unsigned int eip, cs, eflags, useresp, ss; // Pushed by the processor automatically.
};

typedef void (* IRQ__Handler)(IRQ__Regs* Regs);


class IRQ
{
private:
	IRQ__Handler Handlers[256];

public:
	void SetHandler(size_t Index, IRQ__Handler Handler);
	void FireHandler(IRQ__Regs* Regs);

public:
	void Init();

};

// This needs to be declared somewhere else.
extern "C" void WriteByteToPort(int port, int byte);

#endif

