/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	pageallocator.h
	Allocates physical memory pages for use by the system.

******************************************************************************/

#ifndef SORTIX_PAGEALLOCATOR_H
#define SORTIX_PAGEALLOCATOR_H

class PageAllocator
{
private:
	size_t Depth;

private:
	size_t AvailableEnd;
	size_t AvailableRam;
	uint64_t TotalRam;
	uint64_t ReservedRam;
	uint64_t UnaddressableRam;

private:
	size_t NumFreePages;
	size_t NumUsedPages;
	size_t NumPages;

private:
	size_t** Bitmaps;
	size_t* BitmapWidths;
	size_t* BitmapEntries;
	size_t BitmapsSize;
	size_t BitmapLayers;

public:
	void Init(multiboot_info_t* BootInfo, size_t KernelStart, size_t KernelSize);

private:
	void MeasureRam(multiboot_info_t* BootInfo);
	void AllocatePageBitmaps(multiboot_info_t* BootInfo, size_t KernelStart, size_t KernelSize);

private:
	inline void InlineAllocatePage(size_t BitmapLayer, size_t BitNum);
	inline void InlineFreePage(size_t BitmapLayer, size_t BitNum);

public:
	void* AllocatePage();
	size_t AllocatePages(void** Pages, size_t Num);
	void FreePage(void* Page);
	void FreePages(void* Page);

public:
	void IdentityMap();

};

#endif

