/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	panic.cpp
	Displays an error whenever something critical happens.

******************************************************************************/

#include "types.h"
#include "itextscreen.h"
#include "vidmemtextscreen.h"
#include "iprintable.h"
#include "kerneloutput.h"
#include "isr.h"
#include "panic.h"

void PanicHeader(KernelOutput* Out)
{
	Out->SetColor(COLOR8_RED);
	Out->Print("                                                       _                       ");
	Out->Print("                                                      / \\                      ");
	Out->Print("                  /\\    /\\                           /   \\                     ");
	Out->Print("                 /  \\  /  \\                          |   |                     ");
	Out->Print("                /    \\/    \\                         |   |                     ");
	Out->Print("               |  X    X    \\_______________________ /   |                     ");
	Out->Print("               |                                         |                     ");
	Out->Print("               | _________                               /                     ");
	Out->Print("                \\                                       /                      ");
	Out->Print("                  ------       ---------------      ---/                       ");
	Out->Print("                       /       \\             /      \\                          ");
	Out->Print("                      /         \\           /        \\                         ");
	Out->Print("                     /           \\         /          \\                        ");
	Out->Print("                    /_____________\\       /____________\\                       ");
	Out->Print("                                                                               ");
	Out->Print("                                                                               ");
	Out->SetColor((COLOR8_DARK_GREY << 4) + COLOR8_RED);
	Out->Print("                              RED MAXSI OF DEATH                               ");
	Out->SetColor(COLOR8_RED);
	Out->Print("                                                                               ");
}

extern "C" void Panic(const char* Error)
{
	KernelOutput Out;

	PanicHeader(&Out),


	Out.Print(Error);
	Out.Flush();

	HaltKernel();
}

void Panic(const char* Error, const ISR__Regs* Regs)
{
	KernelOutput Out;

	PanicHeader(&Out);

	Out.Print(Error);
	Out.Print("\n");
	Out.PrintF("InterruptNumber=%u, ErrorCode=0x%x\n", Regs->InterruptNumber, Regs->ErrorCode);
	Out.PrintF("edi=0x%x, esi=0x%x, ebp=0x%x, esp=0x%x, ebx=0x%x, edx=0x%x, ecx=0x%x, eax=0x%x\n", Regs->edi, Regs->esi, Regs->ebp, Regs->esp, Regs->ebx, Regs->edx, Regs->ecx, Regs->eax);
	Out.PrintF("eip=0x%x, cs=0x%x, eflags=0x%x, useresp=0x%x, ss=0x%x\n", Regs->eip, Regs->cs, Regs->eflags, Regs->useresp, Regs->ss);
	Out.PrintF("gs=0x%x, fs=0x%x, es=0x%x, ds=0x%x\n", Regs->gs, Regs->fs, Regs->es, Regs->ds);

	Out.Flush();

	HaltKernel();
}

