/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	pit.cpp
	Sets up the Programmable Interval Timer to cause an interrupt every so
	often.

******************************************************************************/

#include "types.h"
#include "irq.h"
#include "pit.h"
#include "timer.h"
#include "kernel.h"

// DEBUG:
#include "panic.h"

#define Frequency 100

void PIT__Interrupt(IRQ__Regs* Regs)
{
	Panic("PIT INTERRUPT!");
	uintmax_t Passed = 1000 / Frequency;
	GTimer->TheFutureIsNow(Passed);
}

void PIT::Init()
{
	return;

	// Register the IRQ callback to our function.
	GIRQ->SetHandler(IRQ0, PIT__Interrupt);

	// The value we send to the PIT is the value to divide it's input clock
	// (1193180 Hz) by, to get our required frequency. Important to note is
	// that the divisor must be small enough to fit into 16-bits.
	uint32_t Divisor = 1193180 / Frequency;

	// Send the command byte.
	WriteByteToPort(0x43, 0x36);
	
	// Divisor has to be sent byte-wise, so split here into upper/lower bytes.
	uint8_t Lower = Divisor & 0xFF;
	uint8_t Higher = ( Divisor >> 8 ) & 0xFF;

	// Send the frequency divisor.
	WriteByteToPort(0x40, Lower);
	WriteByteToPort(0x40, Higher);
}


