/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	kernel.cpp
	A common interface shared by all devices that can be printed text to.

******************************************************************************/

#include "types.h"
#include "globals.h"
#include "iprintable.h"
#include "log.h"
#include "panic.h"
#include "descriptor_tables.h"
#include "timer.h"
#include "multiboot.h"

// TODO: Move these to their own header!
inline void *operator new(size_t, void *p)     { return p; }
inline void *operator new[](size_t, void *p)   { return p; }
inline void  operator delete  (void *, void *) { };
inline void  operator delete[](void *, void *) { };

namespace Sortix
{
	void KernelConstructors()
	{
		GLog = new (GLog) Log;
		GPanic = new (GPanic) PanicClass;
	}

	extern "C" void KernelInit(unsigned long Magic, multiboot_info_t* BootInfo)
	{
		// Initialize the global object's by calling their constructors.
		KernelConstructors();

		// Initialize the kernel log.
		GLog->Init();

		// Write out a sample string
		GLog->Print("Hello, world! This is the Sortix tty driver!\n");

		// Initialise all the ISRs and segmentation
		init_descriptor_tables();

		asm volatile("int $0x3");
		asm volatile("int $0x4");

		init_timer(100);

		int a = 0;

		while ( true  )
		{
			a = a + 1;
		}
	}
}
